package grpc

import (
	"github.com/hidevopsio/hiboot/pkg/factory"
	"github.com/hidevopsio/hiboot/pkg/log"
	"github.com/hidevopsio/hiboot/pkg/utils/reflector"
	"google.golang.org/grpc"
)

// ClientConnector interface is response for creating grpc client connection
type ClientConnector interface {
	Connect(name string, cb interface{}, prop *ClientProperties) (gRpcCli interface{}, err error)
}

type clientConnector struct {
	instantiateFactory factory.InstantiateFactory
}

func newClientConnector(instantiateFactory factory.InstantiateFactory) ClientConnector {
	cc := &clientConnector{
		instantiateFactory: instantiateFactory,
	}
	return cc
}

// Connect connect to grpc server from client
// name: client name
// clientConstructor: client constructor
// properties: properties for configuring
func (c *clientConnector) Connect(name string, clientConstructor interface{}, properties *ClientProperties) (gRpcCli interface{}, err error) {
	host := properties.Host
	if host == "" {
		host = name
	}
	address := host + ":" + properties.Port
	conn := c.instantiateFactory.GetInstance(name)
	if conn == nil {
		// connect to grpc server
		conn, err = grpc.Dial(address, grpc.WithInsecure())
		c.instantiateFactory.SetInstance(name, conn)
		if err == nil {
			log.Infof("gRPC client connected to: %v", address)
		}
	}
	if err == nil && clientConstructor != nil {
		// get return type for register instance name
		gRpcCli, err = reflector.CallFunc(clientConstructor, conn)
	}
	return
}
